/**
 * Copyright (c) 2019 Netmera Research.
 * @author Initial Code
 */

#import "FNetmeraService.h"

@implementation FNetmeraService

static FlutterMethodChannel *_callbackChannel;
static BOOL initialized = NO;
static NSMutableArray *_messageQueue;

NSString *const ON_PUSH_REGISTER = @"onPushRegister";
NSString *const ON_PUSH_RECEIVE = @"onPushReceive";
NSString *const ON_PUSH_RECEIVE_BACKGROUND = @"onPushReceiveBackground";
NSString *const ON_PUSH_OPEN = @"onPushOpen";
NSString *const ON_PUSH_DISMISS = @"onPushDismiss";
NSString *const ON_PUSH_BUTTON_CLICKED = @"onPushButtonClicked";

/// It is called when the Dart callbacks are ready and it creates Netmera Flutter Method Channel.
/// - Parameters:
///   - call: FlutterMethodCall
///   - registrar: FlutterPluginRegistrar
+ (void)initService:(FlutterMethodCall *)call registrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    if (_messageQueue == nil) {
        _messageQueue = [[NSMutableArray alloc] init];
    }
    _callbackChannel = [FlutterMethodChannel methodChannelWithName:@"netmera_flutter_sdk" binaryMessenger:registrar.messenger];
    initialized = YES;
    [self handleMethodCall];
}

/// Sends messages to the Dart side.
/// - Parameter message: Message object
+ (void)sendMessage:(NSDictionary*)message {
    [_callbackChannel invokeMethod:message[@"action"] arguments:message[@"message"]];
}

/// It is called when the Method Channel is ready. Executes pending tasks and removes them from the queue.
+ (void)handleMethodCall {
    @synchronized(self) {
        while ([_messageQueue count] > 0) {
            NSDictionary* message = _messageQueue[0];
            [_messageQueue removeObjectAtIndex:0];
            [self sendMessage: message];
        }
    }
}

/// When a task is requested to be executed, it adds the task to the queue if the Method Channel is not ready.
/// If the channel is ready, it will send directly to the Dart side.
/// - Parameters:
///   - action: action: Name of action
///   - dict: Object to execute
+ (void) handleWork: (NSString*)action dict:(NSDictionary*)dict {
    @synchronized(self) {
        NSDictionary *message = @{
                               @"action": action,
                               @"message": dict
                               };
        if (initialized) {
            [self sendMessage:message];
        } else {
            if (_messageQueue == nil) {
                _messageQueue = [[NSMutableArray alloc] init];
            }
            [_messageQueue addObject:message];
        }
    }
}

@end
